
# Category description for the widget registry

NAME = "ShadowOui PaNOSC Extension"

DESCRIPTION = "Widgets for ShadowOui"

BACKGROUND = "#A9D0F5"

#<div>Icons made by <a href="https://www.flaticon.com/authors/freepik" title="Freepik">Freepik</a> from <a href="https://www.flaticon.com/" title="Flaticon">www.flaticon.com</a></div>
ICON = "icons/eu.png"

PRIORITY = 130
